/*
 * FILE:
 * dwi-run.c
 *
 * FUNCTION:
 * Run the basic DWI interpreter.
 *
 * HISTORY:
 * Linas Vepstas March 2002
 */

#include "config.h"

#include <stdio.h>

#include <glib.h>
#include <glib-object.h>
#include <gnome.h>
#include <glade/glade.h>

#include "perr.h"
#include "readfile.h"
#include "show-message.h"

#ifndef PREFIX
#define PREFIX "/usr/"
#endif

#ifndef SYSCONFDIR
#define SYSCONFDIR "/etc/dwi"
#endif

#ifndef DATADIR
#define DATADIR "/usr/share/dwi"
#endif

#ifndef LIBDIR
#define LIBDIR "/usr/lib"
#endif

/* =================================================================== */

int
main (int argc, char *argv[])
{
	char *dwi_file; 
	DuiInterface *di;
	DuiParser *par;

	if (2 > argc)
	{
		fprintf (stderr, "Usage: %s FILENAME.DUI\n", argv[0]);
		fprintf (stderr, "\tInterpret and run a DWI file\n");
		exit (1);
	} 

	g_type_init ();

	dwi_file = argv[1];

	gnome_program_init ("dwi-run", VERSION, LIBGNOMEUI_MODULE, argc, argv, 
	         GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
	glade_gnome_init();

#define ERR_ONLY	 G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING 
#define DBG_MSG	 ERR_ONLY | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG 

	// #ifndef HAVE_QOF
	g_log_set_handler (G_LOG_DOMAIN, ERR_ONLY, dui_show_message, NULL);
	// #endif /* HAVE_QOF */
	
	di = dui_interface_new();
	par = dui_parser_new(di);
	dwi_gtk_parser_new (par);
	dui_parse (par, dwi_file);
	dui_parser_destroy (par);
	dui_interface_realize (di, TRUE);

	gtk_main ();

	dui_interface_destroy(di);
	printf ("Bye! Please Come Again!\n");

	return 0;
}

/* ========================== END OF FILE ============================ */
