/********************************************************************\
 * parse-qof.c -- Plugin for Dui Parser that handle Qof markup.     *
 * Copyright (C) 2004 Linas Vepstas <linas@linas.org>               *
 * http://dwi.sourceforge.net                                       *
 *                                                                  *
 * This library is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU Lesser General Public       *
 * License as published by the Free Software Foundation; either     *
 * version 2.1 of the License, or (at your option) any later version.
 *                                                                  *
 * This library is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU Lesser General Public License for more details.              *
 *                                                                  *
 * You should have received a copy of the GNU Lesser General Public *
 * License along with this program; if not, contact:                *
 *                                                                  *
 * Free Software Foundation           Voice:  +1-617-542-5942       *
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
 * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
\********************************************************************/

#include "config.h"

#ifdef HAVE_QOF

#include "parse-qof.h"
#include "perr.h"

#include "duifield.h"
#include "parse-utils.h"
#include "readfile.h"

/* ============================================================== */
/* The XML Parsing and parse context routines */

struct DwiQofParseCtxt_s
{
};

/* ============================================================== */

static gboolean
qof_field_handler (gpointer self, DuiField *fld, 
                 const char * row_context, const char **attrs)
{
	DwiQofParseCtxt *ctx = self;
	const char * property=NULL;

	if (NULL == row_context)
	{
		SYNTAX ("qof attribute requies an object type to be sepecified using <row>!");
		return FALSE; 
	}

	int i=0;
	while (attrs[i])
	{
		CASE ("property", property)
      {}
		i += 2;
	}

	// XXX should print warning if no property specified here ... 
	if (property)
	{
   	dui_field_set_qof (fld, row_context, property);
		return TRUE;
	}
	return FALSE;
}

static DuiParserFieldPlugin *
qof_parser_plugin (DwiQofParseCtxt *ctx)
{
	DuiParserFieldPlugin *plg = g_new0 (DuiParserFieldPlugin, 1);

	plg->fieldname = "qof";
	plg->handler = qof_field_handler;
	plg->user_data = ctx;
	
	return plg;
}

/* ============================================================== */

static const char *
qof_row_handler (gpointer self, DuiFieldMap *fm, const char **attrs)
{
	DwiQofParseCtxt *ctx = self;
	const char * qobj_type=NULL;
	const char * property=NULL;

	int i=0;
	while (attrs[i])
	{
		CASE ("qobject",  qobj_type)
		CASE ("property", property)
      {}
		i += 2;
	}
	if (qobj_type)
	{
		dui_field_set_qof_match (&fm->target, qobj_type, property);
	}
	return qobj_type;
}

static DuiParserTablePlugin *
qof_parser_table_plugin (DwiQofParseCtxt *ctx)
{
	DuiParserTablePlugin *plg = g_new0 (DuiParserTablePlugin, 1);

	plg->plugin_name = "qof_parser";
	plg->handler = qof_row_handler;
	plg->user_data = ctx;
	
	return plg;
}

/* ============================================================== */

DwiQofParseCtxt *
qof_parser_new (DuiParser *par)
{
	DwiQofParseCtxt *ctx = g_new0 (DwiQofParseCtxt, 1);

	DuiParserFieldPlugin *plg = qof_parser_plugin(ctx);
	dui_parser_register_field_plugin (par, plg);

	DuiParserTablePlugin *tbl = qof_parser_table_plugin (ctx);
	dui_parser_register_table_plugin (par, tbl);

	return ctx;
}

void 
qof_parser_destroy (DwiQofParseCtxt *ctx)
{
	// XXX mem leak!! free the structures allocated abve! 
	g_free (ctx);
}

#endif /* HAVE_QOF */
/* ======================== END OF FILE ========================= */
/* ============================================================== */
