/*
 * FILE:
 * button-action.c
 *
 * FUNCTION:
 * Example of integration between a pre-existing GTK application, and DWI.
 * 
 * This is a very simple example of how to trigger a DWI form by pressing
 * a button.  It looks up a DWI form from the DWI description file, and then 
 * uses the click of a button to make that form run.
 *
 * HISTORY:
 * Linas Vepstas July 2002
 */

#include "config.h"

#include <stdio.h>

#include <glib-object.h>
#include <gnome.h>
#include <glade/glade.h>

#include "action.h"
#include "interface.h"
#include "show-message.h"
#include "readfile.h"

/* some globals -- makes this example program shorter */
DuiInterface *di = NULL;

/* =================================================================== */

void
button_clicked_cb (GtkButton *button, gpointer x)
{
	DuiAction *act;
	printf ("Button was clicked\n");

	/* DWI "forms" are called "actions" in the C code.  To cause a form
	 * to execute, we need merely to run it; the rest happens automatically.
	 *
	 * The following will cause the form called "search" to be run.  
	 * The definition of this form is in "testbasic.dui", just grep for
	 * 'name="search"' in that file.
	 */
	act = dui_interface_find_action_by_name (di, "search");
	dui_action_run (act);
}

/* =================================================================== */
/* create a window with a button (by hand) */

void
make_window(void)
{
	GtkWidget *main_w, *vbox_w, *button_w;
  
  /* Generic window setup --
   * The next block is plain-old 'typical' GTK setup code */
  main_w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW(main_w), "Integration Demo");
  gtk_widget_set_usize (main_w, 150, 150);
  // gtk_container_border_width(GTK_CONTAINER(main_w),0);

  g_signal_connect (G_OBJECT (main_w), "destroy",
 		     G_CALLBACK (gtk_main_quit), NULL);

  vbox_w = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER(main_w), vbox_w);
  gtk_widget_show (vbox_w);

  button_w = gtk_button_new_with_label ("Press me\nto run search");
  gtk_container_add (GTK_CONTAINER(vbox_w), button_w);
  gtk_widget_show (button_w);

  g_signal_connect (G_OBJECT(button_w), "clicked", 
           G_CALLBACK(button_clicked_cb), NULL);

  gtk_widget_show (main_w);
}

/* =================================================================== */

int
main (int argc, char *argv[])
{
	/* generic gnome/gtk initialization */
	g_type_init ();

	gnome_init ("main", VERSION, argc, argv);
	glade_gnome_init();

	/* Use GUI pops to report error messages */
#define ERR_ONLY	 G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING 
#define DBG_MSG	 ERR_ONLY | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG 
	g_log_set_handler (G_LOG_DOMAIN, ERR_ONLY, dui_show_message, NULL);
	
	/* DUI initialization */
	di = dui_interface_new();
	DuiParser *par = dui_parser_new(di);
	dwi_gtk_parser_new (par);
	dui_parse (par, "testbasic.dui");
	dui_parser_destroy (par);
	dui_interface_realize (di, TRUE);

	/* create a GTK button, hook it to a DWI form. */
	make_window();

	gtk_main ();

	dui_interface_destroy(di);
	printf ("Bye! Please Come Again!\n");

	return 0;
}

/* ========================== END OF FILE ============================ */
