/********************************************************************\
 * duiresolver.c -- Resolve names into actual pointers              *
 * Copyright (C) 2003 Linas Vepstas <linas@linas.org>               *
 * http://dwi.sourceforge.net                                       *
 *                                                                  *
 * This library is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU Lesser General Public       *
 * License as published by the Free Software Foundation; either     *
 * version 2.1 of the License, or (at your option) any later version.
 *                                                                  *
 * This library is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU Lesser General Public License for more details.              *
 *                                                                  *
 * You should have received a copy of the GNU Lesser General Public *
 * License along with this program; if not, contact:                *
 *                                                                  *
 * Free Software Foundation           Voice:  +1-617-542-5942       *
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
 * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
\********************************************************************/

/* duiresolver.c
 * Resolve names into actual pointers to actual objects 
 *
 * Linas Vepstas September 2003
 */

#include <glib.h>
#include "duifield.h"
#include "duiresolver.h"

struct DuiResolverResolver_s {
	DuiResolverFieldFunc resolver_func;
	gpointer user_data;
};

typedef struct DuiResolverResolver_s DuiResolverResolver;

struct DuiResolverRealizer_s {
	DuiResolverRealizeFunc realize_func;
	gpointer user_data;
};

typedef struct DuiResolverRealizer_s DuiResolverRealizer;

/* =================================================================== */

DuiResolver *
dui_resolver_new (void)
{
	DuiResolver *res;
	res = g_new0 (DuiResolver,1);
	res->field_list = NULL;
	res->resolver_list = NULL;
	res->realizer_list = NULL;
	return res;
}

void 
dui_resolver_destroy (DuiResolver *res)
{
	if (!res) return;
	g_list_free (res->field_list);
}

void 
dui_resolver_add_field (DuiResolver *res , DuiField *fld)
{
	if (!res) return;
	res->field_list = g_list_prepend (res->field_list, fld);
}

void 
dui_resolver_add_resolver (DuiResolver *res, 
                           DuiResolverFieldFunc fn, gpointer ud)
{
	if (!res) return;
	DuiResolverResolver *rr = g_new0 (DuiResolverResolver, 1);
	rr->resolver_func = fn;
	rr->user_data = ud;

	res->resolver_list = g_list_prepend (res->resolver_list, rr);
}

void 
dui_resolver_remove_resolver (DuiResolver *res, 
                           DuiResolverFieldFunc fn, gpointer ud)
{
	GList *n;
	if (!res) return;

	for (n=res->resolver_list; n; n=n->next)
	{
		DuiResolverResolver *rr = n->data;
		if ((rr->user_data == ud) && (rr->resolver_func == fn))
		{
			g_free (rr);
			res->resolver_list = g_list_delete_link (res->resolver_list, n);
			return;
		}
	}
}

void 
dui_resolver_resolve (DuiResolver *res)
{
	if (!res) return;

	GList *n, *f;

	for (n=res->resolver_list; n; n=n->next)
	{
		DuiResolverResolver *rr = n->data;

		for (f= res->field_list; f; f=f->next)
		{
			DuiField *fld = f->data;
			(rr->resolver_func) (fld, rr->user_data);
		}
	}
}

/* ================================================================== */

void 
dui_resolver_add_realizer (DuiResolver *res, 
                           DuiResolverRealizeFunc fn, gpointer ud)
{
	if (!res) return;
	DuiResolverRealizer *rr = g_new0 (DuiResolverRealizer, 1);
	rr->realize_func = fn;
	rr->user_data = ud;

	res->realizer_list = g_list_prepend (res->realizer_list, rr);
}

void 
dui_resolver_realize (DuiResolver *res)
{
	if (!res) return;

	GList *n;

	for (n=res->realizer_list; n; n=n->next)
	{
		DuiResolverRealizer *rr = n->data;
		(rr->realize_func) (rr->user_data);
	}
}

/* ======================= END OF FILE =============================== */
